from Framework import Controller
from Framework import templatepage
from Framework.PluginManager import DeclarePlugin
from cherrypy import expose, request  
from ConfigParser import ConfigParser
from QueryResponder import QueryResponder
from cherrypy.lib import static
import os.path
import pickle

class IOVManagement(Controller):
    """Page uses iov_tag.tmpl template
    """
    def __init__(self,context):
        Controller.__init__(self,context,__file__)
        self.__taglist=''
        self.__basepath = __file__.rsplit('/', 1)[0]
        configparser=ConfigParser()
        filename=os.path.join(self.__basepath,'conddbpage.conf')
        configparser.read(filename)
        self.__appurl=configparser.get('APPSERVER','baseurl')
        #template variable dict
        self.ret_dict={}
        self.ret_dict['servicedict']= {}
        self.ret_dict['chosen_det'] = ''
        self.ret_dict['chosen_service'] = ''
        self.ret_dict['chosen_schema'] = ''
        self.ret_dict['viacache'] = ''
        self.ret_dict['tag_list'] = ''
        self.ret_dict['iov_list'] = ''
        self.ret_dict['file'] = ''
        self.ret_dict['error'] = None
        self.ret_dict['path'] = self.__basepath
    @expose    
    def getServiceMap(self):
        """exposed only for debugging purpose
        """
        qResp = QueryResponder(self.__appurl)
        result= qResp.getServiceMap()
        return result
    
    def __buildservicedict(self):
        result={}
        qResp = QueryResponder(self.__appurl)
        r_str=qResp.getServiceMap()
        r=pickle.loads(r_str)
        for (det,value) in r.items():
            result[det]={}
            for v in value:
               [service,schema]=v.split('/')
               if result[det].has_key(service):
                   result[det][service].append(schema)
               else:
                   result[det][service]=[]
                   result[det][service].append(schema)
        return result

    @expose
    def index(self):
        """Top level page. Show clean state
        """
        self.ret_dict['tag_list'] = ''
        self.ret_dict['iov_list'] = ''
        self.ret_dict['file'] = ''
        self.ret_dict['error'] = None
        if len(self.ret_dict['servicedict'].keys()) == 0:
            #print 'index reloading account list'
            self.ret_dict['servicedict']=self.__buildservicedict()
            self.ret_dict['defaultdet'] = self.ret_dict['servicedict'].keys()[0]
            self.ret_dict['defaultservice'] = self.ret_dict['servicedict'][self.ret_dict['defaultdet']].keys()[0]
            self.ret_dict['defaultschema'] = self.ret_dict['servicedict'][self.ret_dict['defaultdet']][self.ret_dict['defaultservice']][0]
        return self.templatePage("iov_tag", self.ret_dict)
    
    @expose
    def get_tags(self, *args, **kwds):
        """Fill in the first form to list all available tags for a given service and schema combination        """
        self.ret_dict['tag_list'] = ''
        self.ret_dict['iov_list'] = ''
        self.ret_dict['error'] = None
        self.ret_dict['chosen_det'] = kwds['det']
        #print 'chosen det ',kwds['det']
        self.ret_dict['chosen_service'] = kwds['service']
        self.ret_dict['chosen_schema'] = kwds['schema']
        self.ret_dict['viacache'] = ''

        self.ret_dict['container']=''
        self.ret_dict['destTag']=''
        self.ret_dict['firstSince']=''
        self.ret_dict['lastTill']=''
        
        if kwds.has_key('viacache') and kwds['viacache']!='':
            self.ret_dict['viacache']='CHECKED'
        qResp = QueryResponder(self.__appurl)
        taglist = qResp.getTags(self.ret_dict['chosen_det'],self.ret_dict['chosen_service'],self.ret_dict['chosen_schema'],self.ret_dict['viacache'])
        for tag in taglist:
            self.ret_dict['tag_list']+='<option style="direction:ltr" value="'+tag+'">'+tag+'</option>'
        if len(self.ret_dict['servicedict'].keys()) == 0:
            #print 'get tags reloading account list'
            self.ret_dict['servicedict']=self.__buildservicedict()
            self.ret_dict['defaultdet'] = self.ret_dict['servicedict'].keys()[0]
            self.ret_dict['defaultservice'] = self.ret_dict['servicedict'][self.ret_dict['defaultdet']].keys()[0]
            self.ret_dict['defaultschema'] = self.ret_dict['servicedict'][self.ret_dict['defaultdet']][self.ret_dict['defaultservice']][0]
        return self.templatePage("iov_tag", self.ret_dict)

    @expose	
    def get_iovs(self, *args, **kwds):
        """fill in the second form to display selected tag info and to save result to file. Can be called after get_tags
        """
        self.ret_dict['tag_list']=self.__taglist
        self.ret_dict['iov_list']=''
        self.ret_dict['container']=''
        self.ret_dict['destTag']=''
        self.ret_dict['firstSince']=''
        self.ret_dict['lastTill']=''
        self.ret_dict['error'] = None        
        if kwds.has_key('sel_tag') is False:
            self.ret_dict['error'] = "Tag has to be selected"
            return self.templatePage("iov_tag", self.ret_dict)
        self.ret_dict['tag_list']=self.__selectTag(kwds['sel_tag'])
        iov_list=self.__prepareIOVList(kwds['det'],kwds['service'],kwds['schema'],kwds['viacache'],kwds['sel_tag'])
        self.ret_dict['iov_list']=iov_list
        if kwds.has_key('file'):           
           qResp = QueryResponder(self.__appurl)
           datafile=qResp.exportIOV(kwds['det'],kwds['service'],kwds['schema'],self.ret_dict['container'],kwds['destTag'],kwds['sel_tag'],kwds['firstSince'],kwds['lastTill'],self.ret_dict['viacache'])
           suggestedfilename='_'.join([self.ret_dict['container'],kwds['sel_tag']])+'.db'
           return static.serve_file(path=datafile,content_type='application/x-download',disposition='attachment',name=suggestedfilename)
       
        if len(self.ret_dict['servicedict'].keys()) == 0:
            #print 'get iovs reloading account list'
            self.ret_dict['servicedict']=self.__buildservicedict()
            self.ret_dict['defaultdet'] = self.ret_dict['servicedict'].keys()[0]
            self.ret_dict['defaultservice'] = self.ret_dict['servicedict'][self.ret_dict['defaultdet']].keys()[0]
            self.ret_dict['defaultschema'] = self.ret_dict['servicedict'][self.ret_dict['defaultdet']][self.ret_dict['defaultservice']][0]
        return self.templatePage("iov_tag", self.ret_dict)

    def __selectTag(self,tag):
        """replace the option list with selected tag highlighted
        """
        sel=self.__taglist
        selected = '<option style="direction:ltr" value="'+tag+'">'+tag+'</option>'
        replacement='<option selected="selected" value="'+tag+'">'+tag+'</option>'
        sel.replace(selected,replacement)
        return sel
    
    def __prepareIOVList(self,det,service,schema,viacache,tag):
        """Get info of selected tag from app server
        """
        qResp = QueryResponder(self.__appurl)
        list = qResp.getIOVForTag(det,service,schema,viacache,tag)
	#list='Tag CSCDBCrosstalk_mc;Time Type 0;PayloadContainerName CSCDBCrosstalkRcd;since \t till \t payloadToken;1 \t  4294967295 \t [DB=7E4ED659-82A4-DA11-A0AC-000E0C5CE043][CNT=HcalGainsRcd][CLID=A069207D-22A2-1BAD-FD51-13613F781A97][TECH=00000B01][OID=00000007-00000000];Total # of payload objects: 1;'	
        fields = list.split(';')
        #pop cup empty 
        fields.pop()
        #number of obj
        lastfiled=fields.pop()
        totalnumber=''.join(lastfiled.split(' ')).split(':')[1]
        self.ret_dict['totalnumber'] = totalnumber
        #Time Type field
        timetype = fields[1].split(' ')[2]
        if timetype=='0':
            timetype='runnumber'
        elif timetype=='1':
            timetype='timestamp'
        elif timetype=='2':
            timetype='lumiid'
        else:
            timetype='unknown'
        self.ret_dict['timetype']=timetype
        #container field
        container = fields[2].split(' ')[1]
        self.ret_dict['container'] = container
        list =''
        for ind, el in enumerate(fields):
            if ind > 3:
                tmp = el.split(' \t ')
                list += '<tr>'
                nfield=0
                for field in tmp:
                    if field!='':
                      nfield+=1
                      if nfield<=2:
                        list += '<td class="iov_cell">'+field+'</td> '
                list+='</tr>'
        return list
		   
DeclarePlugin ("/Controllers/CondDB/IOVManagement", IOVManagement, {"baseUrl": "/IOVManagement"})
		  	
