import os, sys
import cherrypy
from optparse import OptionParser
from Applications import iov,serviceMap
class Root(object):
    @cherrypy.expose
    def index(self):
        """ Top level page
        """
        return "This is Cond DB Application Server"

class Server(object):
    def __init__(self):
        """Constructor
        """
        self.__parser=OptionParser()
        self.__parser.add_option("--basedir",action="store",dest="basedir",
                                 help="application base installation dir default $HOME")
        self.__parser.add_option("--hostname",action="store",dest="hostname",
                                 help="hostname default localhost")
        self.__parser.add_option("-p","--port",action="store",dest="port",
                                 help="port default 8086")       
        self.__parser.add_option("","--cmsswversion",action="store",
                                 dest="cmsswversion", help="cmsswversion(required)")
        self.__platform='slc4_ia32_gcc345'
        
    def __setuprunenv(self,basedir,cmsswversion):
        """Set up cmssw running environment according to installation location and cmssw version
        """
        cmsswlocation=os.path.join(basedir,self.__platform,'cms','cmssw',cmsswversion)
        currentdir=os.path.dirname(os.path.abspath(__file__))
        os.chdir(cmsswlocation)
        stdout_handle = os.popen('scramv1 runtime -sh', 'r')
        text=stdout_handle.read()
        new_string = ''
        for line in text.split('\n'):
            if line.strip():
                new_string += line
        new_string=new_string.replace('export ',' ').strip(';')
        new_string=new_string.replace('; ',' ').strip(' ')
        for env in new_string.split('" '):
            (k,v)=env.split('="')
            if k in ['PATH','LD_LIBRARY_PATH','TNS_ADMIN','ORACLE_HOME','ORA_NLS33','SEAL_PLUGINS','ROOTSYS'] :
                if os.environ.has_key(k):
                    os.environ[k]+=':'+v
                else:
                    os.environ[k]=v
        os.chdir(currentdir)
    def start (self):
        """Start server
        """
        (options, args) = self.__parser.parse_args()
        basedir=os.environ['HOME']
        if options.basedir:
            basedir=options.basedir
        hostname='localhost'
        if options.hostname:
            hostname=options.hostname
        port=8086
        if options.port:
            port=int(options.port)
        cmsswversion=options.cmsswversion.strip(' ')
        self.__setuprunenv(basedir,cmsswversion)
        serverconfig={'global':{'server.socket_host':hostname,'server.socket_port':port,'server.environment':'production'}}
        cherrypy.config.update(serverconfig)
        cherrypy.tree.mount(Root(),script_name='/')
        currentdir=os.path.dirname(os.path.abspath(__file__))
        appconfigfile=os.path.join(currentdir,'config','application.conf')
        print 'appconfigfile',appconfigfile
        cherrypy.tree.mount(iov.IOV(appconfigfile),script_name='/iov')
        detconfigfile=os.path.join(currentdir,'config','detector.conf')
        print 'detconfigfile',detconfigfile
        cherrypy.tree.mount(serviceMap.serviceMap(detconfigfile),script_name='/serviceMap')
        cherrypy.server.quickstart()
        cherrypy.engine.start()
    
if __name__ == '__main__':
    s=Server()
    s.start()
    

